module computer0m(input clock, reset, output [31:0] pc, 
             output [2:0] tick, output [31:0] ir,
				 output [31:0] mar, output [31:0] mdr,
				 inout [31:0] dbus, output m_en, m_rw, 
				 output [7:0] op, output signed [31:0] A, 
				 output signed [31:0] B, output signed [31:0] C, 
				 output [3:0] ra, output [3:0] rb, output [3:0] rc, 
				 output signed [31:0] aluOut);

cpu0m cpu (.clock(clock), .reset(reset), .pc(pc), .tick(tick), .ir(ir), 
.mar(mar), .mdr(mdr), .dbus(dbus), .m_en(m_en), .m_rw(m_rw), 
.op(op), .A(A), .B(B), .C(C), .ra(ra), .rb(rb), .rc(rc), .aluOut(aluOut));
				 
memory0m mem (.clock(clock), .reset(reset), .en(m_en), .rw(m_rw), 
.abus(mar), .dbus_in(mdr), .dbus_out(dbus));

endmodule		 

module cpu0m(input clock, reset, output reg [31:0] pc, 
             output reg [2:0] tick, output reg [31:0] ir,
				 output reg [31:0] mar, output reg [31:0] mdr,
				 inout [31:0] dbus, output reg m_en, m_rw,
				 output reg [7:0] op, output reg signed [31:0] A, 
				 output reg signed [31:0] B, output reg signed [31:0] C, 
				 output reg [3:0] ra, output reg [3:0] rb, output reg [3:0] rc, 
				 output reg signed [31:0] aluOut);
   reg signed [31:0] addr32;
	reg signed [11:0] cx12;
	reg signed [15:0] cx16;
	reg signed [23:0] cx24;
	reg signed [31:0] R [0:15];
	
	`define PC R[15]
				 
	always @(posedge clock) begin
		if (reset) begin
			`PC = 0;
			tick = 0;
		  end
		else begin
			tick = tick+1;
			m_en = 0;
			case (tick)
			// O^q Tick 1..3 G memory.read(m[PC])
				1:	begin  // Tick 1GN PC }׬yƤW
					mar = `PC; // MAR = PC
					m_rw = 1; // m_rw=1 is read mode, read(m[MAR]) => read(m[PC])
					m_en = 1;
					`PC = `PC+4; // PC = PC + 4
				end
				2:	begin  // Tick 2Gir = m[PC]
					mdr = dbus;
					ir = mdr; // IR = dbus = m[PC]
					{op,ra,rb,rc,cx12} = ir;
					cx24 = ir[23:0];
					cx16 = ir[15:0];
				end
				3:	begin  // Tick 3GN PC }׬yƤW
					m_en = 0;
					A = R[ra];
					B = R[rb];
					C = R[rc];
				end
			// O涥q Tick 4..6 G execute(IR)
				4:	begin  // Tick 4GN PC }׬yƤW
					case (op) // Ū OP: Tick 4
						8'h00: begin // OGLD ra, rb, cx ; yNGR[ra] = m[rb+cx]
							mar = B + cx16; // mar = Ū} = R[rb]+cx
							m_rw = 1; // ŪҦGread 
							m_en = 1;
						end
						8'h01: begin // OGST ra, rb, cx ; yNGm[rb+cx]=R[ra]
							mar = B + cx16;  // gJ} = R[rb]+cx
							mdr = A; // gJơGR[ra]
							m_rw = 0; // gJҦGwrite
							m_en = 1;
						end
						8'h13: begin // OGADD ra, rb, rc ; yNGR[ra]=R[rb]+R[rc]
							aluOut = B+C; // [k
						end
						8'h26: begin // OGJMP cx ; yNGPC = PC+cx
							addr32 = cx24; // X cx ର 32 줸
							`PC = `PC+addr32; // DؼЦ}=PC+cx
						end
						default:	begin
						end
					endcase
				end
				5:	begin // Ū OP: Tick 5
					case (op) // LD ra, rb, cx ; R[ra] = R[rb+cx] ; mdr = R[rb+cx]
						8'h00: begin // OGLD ra, rb, cx ; yNGR[ra] = m[rb+cx]
							mdr = dbus; // oOǦ^ dbus = m[rb+cx]
							R[ra] = mdr; // sJؼмȦs R[ra] 
							A = mdr;
							m_en = 0; // Ū
						end
						8'h01: begin // OGST ra, rb, cx ; yNGm[rb+cx]=R[ra]
							m_en = 0; // gJ
						end
						8'h13: begin // OGADD ra, rb, rc ; yNGR[ra]=R[rb]+R[rc]
							R[ra] = aluOut;
						end
						8'h26: begin // OGJMP cx ; yNGPC = PC+cx
						end
						default:	begin
						end
					endcase
				end				
				6:	begin
					tick = 0;
				end
				default:	begin
				end
			endcase
		end
		pc = `PC;
	end
endmodule

module memory0m(input clock, reset, en, rw, 
			input [31:0] abus, input [31:0] dbus_in, output [31:0] dbus_out);
reg [7:0] m [0:128];
reg [31:0] data;

	always @(clock or reset or abus or en or rw or dbus_in) 
	begin
		if (reset == 1) begin		
			{m[0],m[1],m[2],m[3]}    = 32'h001F0018; // 0000       LD   R1, K1
			{m[4],m[5],m[6],m[7]}    = 32'h002F0010; // 0004       LD   R2, K0
			{m[8],m[9],m[10],m[11]}  = 32'h003F0014; // 0008       LD   R3, SUM
			{m[12],m[13],m[14],m[15]}= 32'h13221000; // 000C LOOP: ADD  R2, R2, R1
			{m[16],m[17],m[18],m[19]}= 32'h13332000; // 0010       ADD  R3, R3, R2
			{m[20],m[21],m[22],m[23]}= 32'h26FFFFF4; // 0014       JMP  LOOP
			{m[24],m[25],m[26],m[27]}= 32'h00000000; // 0018 K0:   WORD 0
			{m[28],m[29],m[30],m[31]}= 32'h00000001; // 001C K1:   WORD 1
			{m[32],m[33],m[34],m[35]}= 32'h00000000; // 0020 SUM:  WORD 0
			data = 32'hZZZZZZZZ; 						
		end else if (abus >=0 && abus < 125) begin
			if (en == 1 && rw == 0) begin // r_w==0:write
				data = dbus_in;
				{m[abus], m[abus+1], m[abus+2], m[abus+3]} = dbus_in;
			end
			else if (en == 1 && rw == 1) // r_w==1:read
				data = {m[abus], m[abus+1], m[abus+2], m[abus+3]};
			else
				data = 32'hZZZZZZZZ;
		end else
			data = 32'hZZZZZZZZ;
	end
	assign dbus_out = data;
endmodule
