module cpu0m(input clock, reset, output reg [31:0] pc, 
             output reg [2:0] tick, output reg [31:0] ir,
				 output reg [31:0] mar, output reg [31:0] mdr,
				 inout [31:0] dbus, output reg m_en, m_rw);

cpu0m DUT (.clock(clock), .reset(reset), .pc(pc), .tick(tick), .ir(ir), 
.mar(mar), .mdr(mdr), .dbus(dbus), .m_en(m_en), .m_rw(m_rw));
				 
	always @(posedge clock) begin
		if (reset) 
		  begin
			pc = 0;
			tick = 0;
			m_en = 1;
		  end
		else begin
			tick = tick+1;
			case (tick)
				1:
				begin // memory.read(m[PC])
					mar = pc; // MAR = PC
					m_rw = 1; // m_rw=1 is read mode, read(m[MAR]) => read(m[PC])
				end
				2: 
				begin
					mdr = dbus;
					ir = mdr; // IR = dbus = m[PC]
				end
				3:
				begin
					pc = pc+1;
				end
				4:
				begin
				end
				5:
				begin
				end
				6:
				begin
					tick = 0;
				end
				default:
				begin
				end
			endcase
		end
	end
				 
endmodule

module memory0m(input clock, reset, en, rw, 
			input [31:0] abus, input [31:0] dbus_in, output [31:0] dbus_out);
reg [31:0] m [0:128];
reg [31:0] data;

	always @(clock or reset or abus or en or rw or dbus_in) 
	begin
		if (reset == 1) begin
			m[0] <= 32'h00100010;
			m[1] <= 32'h0110000C; 
			m[2] <= 32'h2C000000;  
			m[3] <= 32'h00000000; 
			m[4] <= 32'h0000001D; 
			data <= 32'hZZZZZZZZ;
		end else if (abus >=0 && abus < 128) begin
			if (en == 1 && rw == 0) // r_w==0:write
			begin
				data <= dbus_in;
				m[abus] <= dbus_in;
			end
			else if (en == 1 && rw == 1) // r_w==1:read
				data <= m[abus];
			else
				data <= 32'hZZZZZZZZ;
		end else
			data <= 32'hZZZZZZZZ;
	end
	assign dbus_out = data;
endmodule


